#include "StdAfx.h"
#include "ChatItem.h"


#include "Resource.h"
/*
 * @note:
 * 1: append the file transfer control items.
 */
CChatItem::CChatItem(const ChatItemInfo& ChatItemInfo)
{
	m_stItemInfo = ChatItemInfo;
	m_nFailCount = 0;
	m_bGroupUserChat = FALSE;
	m_nSucceedCount = 0;
	m_strToolTip.Empty();
	m_nUserStatus = ustOFFLINE;
}

CChatItem::~CChatItem(void)
{
}

void CChatItem::SetMsg(LPCTSTR lpsgMsg)
{
	m_stItemInfo.strMsg = lpsgMsg;
}



void CChatItem::SetGroupUserChat(BOOL bVal)
{
	m_bGroupUserChat = bVal;
}

BOOL CChatItem::IsGroupUserChat()
{
	return m_bGroupUserChat;
}

FontStyle CChatItem::GetFontStyle()
{
	FontStyle style = FontStyleRegular;

	if ((m_stItemInfo.cf.dwMask & CFM_BOLD) && (m_stItemInfo.cf.dwEffects & CFE_BOLD))
		style = (FontStyle)(style | FontStyleBold);
	
	if ((m_stItemInfo.cf.dwMask & CFM_ITALIC) && (m_stItemInfo.cf.dwEffects & CFE_ITALIC))
		style = (FontStyle)(style | FontStyleItalic);
	
	if ((m_stItemInfo.cf.dwMask & CFM_UNDERLINE) && (m_stItemInfo.cf.dwEffects & CFE_UNDERLINE))
		style = (FontStyle)(style | FontStyleUnderline);
	
	if ((m_stItemInfo.cf.dwMask & CFM_STRIKEOUT) && (m_stItemInfo.cf.dwEffects & CFE_STRIKEOUT))
		style = (FontStyle)(style | FontStyleStrikeout);

	return style;
}

REAL CChatItem::GetFontHeight()
{
	REAL nFontHeight = (REAL)(m_stItemInfo.cf.yHeight / 20.0);

	if (nFontHeight < DEF_CHAT_MSG_MIN_FONT_SIZE)
		nFontHeight = DEF_CHAT_MSG_MIN_FONT_SIZE;						

	return nFontHeight;
}

WCHAR* CChatItem::GetFontFaceName()
{
	return m_stItemInfo.cf.szFaceName;
}

CString CChatItem::GetFailCount()
{
	CString strRet;
	strRet.Format(_T("%d"), m_nFailCount);

	return strRet;
}

CString CChatItem::GetUserId()
{
	return m_stItemInfo.strUserID;
}

BOOL CChatItem::IsFailedMsg()
{
	BOOL bResult = TRUE;
	if (m_stItemInfo.eMsgGB == CHAT_MSG_CHAT || m_stItemInfo.eMsgGB == CHAT_MSG_STICKER) 
	{
		if (m_stItemInfo.nMsgStatus == CMS_SUCCEED)
		{
			bResult = FALSE;
		}
	}

	return bResult;
}

int CChatItem::IncreaseFailCount()
{
	m_nFailCount++;

	return m_nFailCount;
}


BOOL CChatItem::IsTalkType()
{
	if (m_stItemInfo.eMsgGB != CHAT_MSG_SYS && m_stItemInfo.eMsgGB == CHAT_MSG_NONE )
	{
		return TRUE;
	}
	return FALSE;
}

BOOL CChatItem::IsSameUser( LPCTSTR strUserID )
{
	if (m_stItemInfo.strUserID.CompareNoCase(strUserID) == 0)
	{
		return TRUE;
	}
	return FALSE;
}

BOOL CChatItem::IsSysType()
{
	return m_stItemInfo.eMsgGB == CHAT_MSG_SYS;
}	


BOOL CChatItem::IsJoinOrLeaveType()
{
	return (m_stItemInfo.eMsgGB == CHAT_MSG_JOIN) || (m_stItemInfo.eMsgGB == CHAT_MSG_LEAVE);
}

BOOL CChatItem::IsNullMsg()
{
	if (m_stItemInfo.strMsg.IsEmpty())
	{
		return TRUE;
	}
	return FALSE;
}

CString CChatItem::GetNameSelectedText()
{
    if (m_NameSelectRange.IsValid())
	{
		return GetChatItemInfo()->strUserName.Mid(m_NameSelectRange.nStart, m_NameSelectRange.nEnd - m_NameSelectRange.nStart + 1);
	}

	return _T("");
}

CString CChatItem::GetTimeSelectedText(CString strTime)
{
   if (m_TimeSelectRange.IsValid())
	{
		return strTime.Mid(m_TimeSelectRange.nStart, m_TimeSelectRange.nEnd - m_TimeSelectRange.nStart + 1);
	}
   return _T("");
}

void CChatItem::SelectAll(CString strTime)
{
	m_TimeSelectRange.nStart = 0;
	m_TimeSelectRange.nEnd =  strTime.GetLength() -1;
	m_NameSelectRange.nStart = 0;
	m_NameSelectRange.nEnd = GetChatItemInfo()->strUserName.GetLength() - 1;
}

CString CChatItem::GetNewMessage()
{
	return m_stItemInfo.strMsg;
}

void CChatItem::SetToolTip( CString strToolTip )
{
	m_strToolTip = strToolTip;
}

CString CChatItem::GetToolTip()
{
	//return m_strToolTip;
	if(m_strUserEmail.IsEmpty())
	{
		const NE_MEMBER_DATA* pData = g_pMessengerActor->GetEmployeeInfo(m_stItemInfo.strUserID);
		if(pData != NULL)
		{
			m_strUserEmail = pData->strEmail.c_str();
		}
	}
	CString strToolTip;
	strToolTip.Format(LoadStringFromResource(IDS_STRING_RECENT_CHAT_TOOLTIP_3), 
		m_stItemInfo.strUserName, GetPresenceTip(m_nUserStatus), m_strUserEmail);

	return strToolTip;
}

void CChatItem::UpdateUserStatus( GTUserStatusType status )
{
	m_nUserStatus = status;
}

GTUserStatusType CChatItem::GetUserStatus()
{
	return m_nUserStatus;
}

void CChatItem::SetUserEmail( CString strEmail )
{
	m_strUserEmail = strEmail;
}

CString CChatItem::GetUserEmail()
{
	return m_strUserEmail;
}


CChatMsgItem::CChatMsgItem( const ChatItemInfo& ChatItemInfo ) : CChatItem(ChatItemInfo)
{
	_ParseUrlInMsg(ChatItemInfo.strMsg);
}

CChatMsgItem::~CChatMsgItem()
{
	m_UrlPosVector.clear();
}

CString CChatMsgItem::GetSelectedText()
{
	if (m_SelectRange.IsValid())
	{
		return GetChatItemInfo()->strMsg.Mid(m_SelectRange.nStart, m_SelectRange.nEnd - m_SelectRange.nStart + 1);
	}
	return __super::GetSelectedText();
}

void CChatMsgItem::_ParseUrlInMsg( LPCTSTR strMsg )
{
	CString strTmpMsg = strMsg;
	strTmpMsg.MakeLower();

	TextRange urlPos;
	const UINT uKeywords = 8;
	CString strKeyword[uKeywords]= {_T("http://"), 
		_T("https://"), 
		_T("ftp://"),
		_T("www."), 
		_T(".co.kr"),
		_T(".com"), 
		_T(".net"), 
		_T(".org")};

	//int nLength = strTmpMsg.GetLength();

	CString strTokenize;
	CString strDelimeter(_T("\" \' \r \n \t ,"));
	int nCurPos = 0;

	strTokenize = strTmpMsg.Tokenize(strDelimeter, nCurPos);
	while(strTokenize != _T(""))
	{
		for (UINT i = 0; i < uKeywords; i++)
		{
			int nFind = strTokenize.Find(strKeyword[i]);

			if (nFind > -1)
			{
				urlPos.nStart = nCurPos - strTokenize.GetLength() - 1;
				urlPos.nEnd = nCurPos - 1;
				m_UrlPosVector.push_back(urlPos);
				if (i < 3)
				{
					break;
				}
			}
		}
		strTokenize = strTmpMsg.Tokenize(strDelimeter, nCurPos);
	}
}

void CChatMsgItem::GetUrlVector( std::vector<TextRange>& vcUrl )
{
	for (size_t i = 0; i < m_UrlPosVector.size(); i++)
	{
		vcUrl.push_back(m_UrlPosVector[i]);
	}
}

LPTextRange CChatMsgItem::GetSelectRange()
{
	return &m_SelectRange;
}

CString CChatMsgItem::GetUrlString( int nIndex )
{
	CString strUrl = _T("");
	for (size_t i = 0; i < m_UrlPosVector.size(); i++)
	{
		if (m_UrlPosVector[i].nStart <= nIndex && nIndex <= m_UrlPosVector[i].nEnd)
		{
			strUrl = m_stItemInfo.strMsg.Mid(m_UrlPosVector[i].nStart, m_UrlPosVector[i].nEnd - m_UrlPosVector[i].nStart);
			break;
		}
	}
	return strUrl;
}

BOOL CChatMsgItem::FindString( LPCTSTR strSearchText )
{
	CString str = strSearchText;
	int nPos = m_stItemInfo.strMsg.Find(strSearchText);
	if (nPos != -1)
	{
		m_SelectRange.Reset();
		m_SelectRange.nStart = nPos;
		m_SelectRange.nEnd = nPos + str.GetLength() - 1;
		return TRUE;
	}
	return FALSE;
}

void CChatMsgItem::SelectAll(CString strTime)
{
	CChatItem::SelectAll(strTime);
	m_SelectRange.nStart = 0;
	m_SelectRange.nEnd = GetChatItemInfo()->strMsg.GetLength() - 1;
}


CString CChatStickerItem::GetSelectedText()
{
	CString strSticker = _T("");
	if(m_bSelected)
	{
	   strSticker.Format(_T("(%s)"), LoadStringFromResource(IDS_STRING377));
	}
	
	return strSticker;
}

void CChatStickerItem::SelectAll( CString strTime )
{
	CChatItem::SelectAll(strTime);
	m_bSelected = TRUE;
}

CString CChatStickerItem::GetNewMessage()
{
	return LoadStringFromResource(IDS_STICKER_NEW_MSG);
}

void CChatFileItem::SelectAll( CString strTime )
{
	CChatItem::SelectAll(strTime);
}

CString CChatFileItem::GetNewMessage()
{
	return m_pTranslateFileInfo->m_fileInfo.fileName.c_str();
}

void CChatFileItem::SetMsgStatus(int nMsgStatus)
{
	CChatItem::SetMsgStatus(nMsgStatus);
	m_pTranslateFileInfo->SetStatus((TransferStatus)nMsgStatus);
}

BOOL CChatFileItem::IsFailedMsg()
{
	BOOL bResult = (m_pTranslateFileInfo->GetStatus() == sUploadSuccess)?FALSE:TRUE;
	return bResult;
}

CString CChatFileItem::GetUserId()
{
	return m_pTranslateFileInfo->m_strSenderID;
}

int CChatFileItem::GetMsgStatus()		
{
	return m_pTranslateFileInfo->GetStatus();
}	

IMPLEMENT_SERIAL(stu_ChatItemInfo, CObject, 1)

CString CChatUserJoinItem::GetNewMessage()
{
	CString strNewMessage;
	strNewMessage.Format(_T("%s%s"), m_stItemInfo.strUserName, m_stItemInfo.strMsg);
	return strNewMessage;
}

CChatUserJoinItem::CChatUserJoinItem( const ChatItemInfo& ChatItemInfo ) 
: CChatItem(ChatItemInfo)
{
	if(ChatItemInfo.eMsgGB == CHAT_MSG_JOIN)
	{
		m_bUserJoin = TRUE;
	}
	else
	{
		m_bUserJoin = FALSE;
	}
}

CString CChatUserJoinItem::GetUserName()
{
	return m_stItemInfo.strUserName;
}

BOOL CChatUserJoinItem::IsUserJoin()
{
	return m_bUserJoin;
}
